import 'package:flutterx_http/src/adapter/adapter.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

class LoggableInterceptor with Loggable implements XHttpInterceptor {
  @override
  Future<XHttpResponse> intercept(XHttpRequest request, XHttpRequestOptions options, InterceptorChain next) async {
    logInfo('HTTP Request ${request.method} ${request.url}', () {
      _logKV('method', request.method);
      _logKV('url', '${request.url.scheme}://${request.url.host}:${request.url.port}${request.url.path}');
      final parameters = request.url.queryParameters;
      if (parameters.isNotEmpty) logDebug('queryParameters:', () => parameters.forEach(_logKV));
      if (request.headers.isNotEmpty) logDebug('headers:', () => request.headers.forEach(_logKV));
      final contentLength = request.body?.contentLength;
      if (contentLength != null) _logKV('contentLength', contentLength);
    });
    final response = await next(request, options);
    logInfo('HTTP Response ${response.request.method} ${response.request.url}', () {
      if (response.headers.isNotEmpty) logDebug('headers:', () => response.headers.forEach(_logKV));
      _logKV('statusCode', response.statusCode, level: LogLevel.info);
      if (response.statusMessage.isNotEmpty) _logKV('statusMessage', response.statusMessage, level: LogLevel.info);
      final contentLength = request.body?.contentLength;
      if (contentLength != null) _logKV('contentLength', contentLength);
    });
    return response;
  }

  void _logKV(dynamic key, dynamic value, {LogLevel level = LogLevel.debug}) => log(level, '$key: $value');
}
