import 'dart:async';
import 'dart:convert';
import 'dart:typed_data';

import 'package:flutterx_http/flutterx_http.dart';
import 'package:flutterx_utils/flutterx_utils.dart';

extension ResponseConverter on Future<XHttpResponse> {
  Future<Uint8List> asBytes() => then((response) => response.body.asBytes());

  Future<Stream<List<int>>> asStream() => then((response) => response.body.asStream());

  Future<String> asText() => then((response) => response.body.asBytes().then(response.encoding.decode));

  Future<T> asJson<T>(FromJson<T> fromJson) => asText().then((json) => fromJson(jsonDecode(json)));

  Future<T> asJsonObject<T>(FromJsonObject<T> fromJson) => asJson((json) => fromJson(json));

  Future<List<T>> asJsonList<T>(FromJsonObject<T> fromJson) => asJson((json) => fromJsonList(json, fromJson));

  Future<Map<String, T>> asJsonMap<T>(FromJsonObject<T> fromJson) => asJson((json) => fromJsonMap(json, fromJson));
}
